#include "stdafx.h"
#include "windows.h"

struct BuilderSZ
{
	char* string;
	char* current;
	unsigned long length;
	char* _sz0;
	char* _sz1;
	unsigned long _cap;
	unsigned long _cur;
	unsigned long makespace;
};

void bsz_init(struct BuilderSZ* bsz)
{
	bsz->string = 0;
	bsz->current = 0;
	bsz->length = 0;
	bsz->_sz0 = 0;
	bsz->_sz1 = 0;
	bsz->_cap = 0;
	bsz->_cur = 0;
	bsz->makespace = 0;
}

void bsz_increaseAllocation(struct BuilderSZ* bsz, const unsigned long len)
{
    if ((len + bsz->length + 1) > bsz->_cap)
	{
        bsz->_cap = (((len + bsz->length) * 2) + 1);
        
        if (bsz->_cur == 0)
		{
            bsz->_sz1 = (char*)malloc(bsz->_cap * sizeof(char));
            memcpy(bsz->_sz1, bsz->_sz0, bsz->length);
            bsz->string = bsz->_sz1;
            free(bsz->_sz0);
            bsz->_cur = 1;
		}
		else
		{
            bsz->_sz0 = (char*)malloc(bsz->_cap * sizeof(char));
            memcpy(bsz->_sz0, bsz->_sz1, bsz->length);
            bsz->string = bsz->_sz0;
            free(bsz->_sz1);
            bsz->_cur = 0;
		}
	}
}

void bsz_free(struct BuilderSZ* bsz)
{
	if (bsz->string != 0)
	{
		if (bsz->_cur == 0)
		{
			free(bsz->_sz0);
		}
		else
		{
			free(bsz->_sz1);
		}

		bsz_init(bsz);
	}
}

void bsz_clear(struct BuilderSZ* bsz)
{
	bsz_free(bsz);
}

void bsz_sz_n(struct BuilderSZ* bsz, char* szBuffer, unsigned long len)
{
	bsz_increaseAllocation(bsz, len);
	lstrcpy(&bsz->string[bsz->length], szBuffer);
	bsz->length += len;
	bsz->string[bsz->length] = '\0';
	bsz->current = &bsz->string[bsz->length];
}

void bsz_sz_bool(struct BuilderSZ* bsz, BOOL value, char* trueString, char* falseString)
{
	unsigned long len = 0;
	
	if (value)
	{
		len = lstrlen(trueString);
		bsz_sz_n(bsz, trueString, len);
	}
	else
	{
		len = lstrlen(falseString);
		bsz_sz_n(bsz, falseString, len);
	}
}

void bsz_rt(struct BuilderSZ* bsz)
{
	bsz_sz_n(bsz, "\r\n", 2);
}

void bsz_sz(struct BuilderSZ* bsz, char* szBuffer)
{
	bsz_sz_n(bsz, szBuffer, lstrlen(szBuffer));
}

void bsz_ul(struct BuilderSZ* bsz, unsigned long value)
{
	char szBuffer[16];

	ultoa(value, szBuffer, 10);
	bsz_sz_n(bsz, szBuffer, lstrlen(szBuffer));
}

void bsz_sl(struct BuilderSZ* bsz, signed long value)
{
	char szBuffer[16];

	ltoa(value, szBuffer, 10);
	bsz_sz_n(bsz, szBuffer, lstrlen(szBuffer));
}

void bsz_makespace(struct BuilderSZ* bsz, unsigned long spaceCount)
{
	bsz->makespace = spaceCount;
	bsz_increaseAllocation(bsz, spaceCount);
	bsz->current = &bsz->string[bsz->length];
}

void bsz_fixupZeroTerm(struct BuilderSZ* bsz)
{
	bsz->length += lstrlen(&bsz->string[bsz->length]);
}

void bsz_fixupByBuffer(struct BuilderSZ* bsz, unsigned long len)
{
	bsz->length += len;
}

int bsz_compare(struct BuilderSZ* bsz1, struct BuilderSZ* bsz2, BOOL caseSensetive)
{
	if (caseSensetive)
		return lstrcmpi(bsz1->string, bsz2->string);
	else
		return lstrcmp(bsz1->string, bsz2->string);
}

BOOL bsz_equals(struct BuilderSZ* bsz1, struct BuilderSZ* bsz2, BOOL caseSensetive)
{
	if (bsz1->length != bsz2->length)
	{
		return FALSE;
	}
	else
	{
		if (caseSensetive)
		{
			if (lstrcmpi(bsz1->string, bsz2->string) == 0)
			{	
				return TRUE;
			}
			else
			{
				return FALSE;
			}
		}
		else
		{
			if (lstrcmp(bsz1->string, bsz2->string) == 0)
			{	
				return TRUE;
			}
			else
			{
				return FALSE;
			}
		}
	}
}

void bsz_LoadResourceString(struct BuilderSZ* bsz, HINSTANCE hInstance, UINT uID, int nBufferMax)
{
   char* szBuffer = (char*)malloc(nBufferMax + 1);

    ZeroMemory(szBuffer, nBufferMax);
	LoadString(hInstance == NULL ? GetModuleHandle(NULL) : hInstance, uID, szBuffer, nBufferMax);
	bsz_sz(bsz, szBuffer);
	free(szBuffer);
}

void bsz_copy_bsz(struct BuilderSZ* source, struct BuilderSZ* destin)
{
	bsz_free(destin);
	bsz_sz_n(destin, source->string, source->length);
}

void bsz_tolower(struct BuilderSZ* bsz)
{
	if (bsz->length > 0)
    {
		#pragma warning(disable: 4996)
        _strlwr((char*)bsz->string);
        #pragma warning(default: 4996)
    }
}

void bsz_toupper(struct BuilderSZ* bsz)
{
	if (bsz->length > 0)
    {
        #pragma warning(disable: 4996)
        _strupr((char*)bsz->string);
        #pragma warning(default: 4996)
    }
}
